# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import (
    global_ip_assignment_list_params,
    global_ip_assignment_create_params,
    global_ip_assignment_update_params,
)
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.global_ip_assignment_list_response import GlobalIPAssignmentListResponse
from ..types.global_ip_assignment_create_response import GlobalIPAssignmentCreateResponse
from ..types.global_ip_assignment_delete_response import GlobalIPAssignmentDeleteResponse
from ..types.global_ip_assignment_update_response import GlobalIPAssignmentUpdateResponse
from ..types.global_ip_assignment_retrieve_response import GlobalIPAssignmentRetrieveResponse

__all__ = ["GlobalIPAssignmentsResource", "AsyncGlobalIPAssignmentsResource"]


class GlobalIPAssignmentsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> GlobalIPAssignmentsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return GlobalIPAssignmentsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> GlobalIPAssignmentsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return GlobalIPAssignmentsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        global_ip_id: str | Omit = omit,
        is_in_maintenance: bool | Omit = omit,
        wireguard_peer_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> GlobalIPAssignmentCreateResponse:
        """
        Create a Global IP assignment.

        Args:
          global_ip_id: Global IP ID.

          is_in_maintenance: Enable/disable BGP announcement.

          wireguard_peer_id: Wireguard peer ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/global_ip_assignments",
            body=maybe_transform(
                {
                    "global_ip_id": global_ip_id,
                    "is_in_maintenance": is_in_maintenance,
                    "wireguard_peer_id": wireguard_peer_id,
                },
                global_ip_assignment_create_params.GlobalIPAssignmentCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=GlobalIPAssignmentCreateResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> GlobalIPAssignmentRetrieveResponse:
        """
        Retrieve a Global IP assignment.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/global_ip_assignments/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=GlobalIPAssignmentRetrieveResponse,
        )

    def update(
        self,
        id: str,
        *,
        body: global_ip_assignment_update_params.Body,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> GlobalIPAssignmentUpdateResponse:
        """
        Update a Global IP assignment.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._patch(
            f"/global_ip_assignments/{id}",
            body=maybe_transform(body, global_ip_assignment_update_params.GlobalIPAssignmentUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=GlobalIPAssignmentUpdateResponse,
        )

    def list(
        self,
        *,
        page: global_ip_assignment_list_params.Page | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> GlobalIPAssignmentListResponse:
        """
        List all Global IP assignments.

        Args:
          page: Consolidated page parameter (deepObject style). Originally: page[number],
              page[size]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/global_ip_assignments",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"page": page}, global_ip_assignment_list_params.GlobalIPAssignmentListParams),
            ),
            cast_to=GlobalIPAssignmentListResponse,
        )

    def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> GlobalIPAssignmentDeleteResponse:
        """
        Delete a Global IP assignment.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/global_ip_assignments/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=GlobalIPAssignmentDeleteResponse,
        )


class AsyncGlobalIPAssignmentsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncGlobalIPAssignmentsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncGlobalIPAssignmentsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncGlobalIPAssignmentsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncGlobalIPAssignmentsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        global_ip_id: str | Omit = omit,
        is_in_maintenance: bool | Omit = omit,
        wireguard_peer_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> GlobalIPAssignmentCreateResponse:
        """
        Create a Global IP assignment.

        Args:
          global_ip_id: Global IP ID.

          is_in_maintenance: Enable/disable BGP announcement.

          wireguard_peer_id: Wireguard peer ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/global_ip_assignments",
            body=await async_maybe_transform(
                {
                    "global_ip_id": global_ip_id,
                    "is_in_maintenance": is_in_maintenance,
                    "wireguard_peer_id": wireguard_peer_id,
                },
                global_ip_assignment_create_params.GlobalIPAssignmentCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=GlobalIPAssignmentCreateResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> GlobalIPAssignmentRetrieveResponse:
        """
        Retrieve a Global IP assignment.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/global_ip_assignments/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=GlobalIPAssignmentRetrieveResponse,
        )

    async def update(
        self,
        id: str,
        *,
        body: global_ip_assignment_update_params.Body,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> GlobalIPAssignmentUpdateResponse:
        """
        Update a Global IP assignment.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._patch(
            f"/global_ip_assignments/{id}",
            body=await async_maybe_transform(body, global_ip_assignment_update_params.GlobalIPAssignmentUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=GlobalIPAssignmentUpdateResponse,
        )

    async def list(
        self,
        *,
        page: global_ip_assignment_list_params.Page | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> GlobalIPAssignmentListResponse:
        """
        List all Global IP assignments.

        Args:
          page: Consolidated page parameter (deepObject style). Originally: page[number],
              page[size]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/global_ip_assignments",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"page": page}, global_ip_assignment_list_params.GlobalIPAssignmentListParams
                ),
            ),
            cast_to=GlobalIPAssignmentListResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> GlobalIPAssignmentDeleteResponse:
        """
        Delete a Global IP assignment.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/global_ip_assignments/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=GlobalIPAssignmentDeleteResponse,
        )


class GlobalIPAssignmentsResourceWithRawResponse:
    def __init__(self, global_ip_assignments: GlobalIPAssignmentsResource) -> None:
        self._global_ip_assignments = global_ip_assignments

        self.create = to_raw_response_wrapper(
            global_ip_assignments.create,
        )
        self.retrieve = to_raw_response_wrapper(
            global_ip_assignments.retrieve,
        )
        self.update = to_raw_response_wrapper(
            global_ip_assignments.update,
        )
        self.list = to_raw_response_wrapper(
            global_ip_assignments.list,
        )
        self.delete = to_raw_response_wrapper(
            global_ip_assignments.delete,
        )


class AsyncGlobalIPAssignmentsResourceWithRawResponse:
    def __init__(self, global_ip_assignments: AsyncGlobalIPAssignmentsResource) -> None:
        self._global_ip_assignments = global_ip_assignments

        self.create = async_to_raw_response_wrapper(
            global_ip_assignments.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            global_ip_assignments.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            global_ip_assignments.update,
        )
        self.list = async_to_raw_response_wrapper(
            global_ip_assignments.list,
        )
        self.delete = async_to_raw_response_wrapper(
            global_ip_assignments.delete,
        )


class GlobalIPAssignmentsResourceWithStreamingResponse:
    def __init__(self, global_ip_assignments: GlobalIPAssignmentsResource) -> None:
        self._global_ip_assignments = global_ip_assignments

        self.create = to_streamed_response_wrapper(
            global_ip_assignments.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            global_ip_assignments.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            global_ip_assignments.update,
        )
        self.list = to_streamed_response_wrapper(
            global_ip_assignments.list,
        )
        self.delete = to_streamed_response_wrapper(
            global_ip_assignments.delete,
        )


class AsyncGlobalIPAssignmentsResourceWithStreamingResponse:
    def __init__(self, global_ip_assignments: AsyncGlobalIPAssignmentsResource) -> None:
        self._global_ip_assignments = global_ip_assignments

        self.create = async_to_streamed_response_wrapper(
            global_ip_assignments.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            global_ip_assignments.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            global_ip_assignments.update,
        )
        self.list = async_to_streamed_response_wrapper(
            global_ip_assignments.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            global_ip_assignments.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ...._models import BaseModel

__all__ = ["FineTuningJob", "Hyperparameters"]


class Hyperparameters(BaseModel):
    n_epochs: int
    """The number of epochs to train the model for.

    An epoch refers to one full cycle through the training dataset.
    """


class FineTuningJob(BaseModel):
    id: str
    """The name of the fine-tuned model that is being created."""

    created_at: int
    """The Unix timestamp (in seconds) for when the fine-tuning job was created."""

    finished_at: Optional[int] = None
    """The Unix timestamp (in seconds) for when the fine-tuning job was finished.

    The value will be null if the fine-tuning job is still running.
    """

    hyperparameters: Hyperparameters
    """The hyperparameters used for the fine-tuning job."""

    model: str
    """The base model that is being fine-tuned."""

    organization_id: str
    """The organization that owns the fine-tuning job."""

    status: Literal["queued", "running", "succeeded", "failed", "cancelled"]
    """The current status of the fine-tuning job."""

    trained_tokens: Optional[int] = None
    """The total number of billable tokens processed by this fine-tuning job.

    The value will be null if the fine-tuning job is still running.
    """

    training_file: str
    """The storage bucket or object used for training."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .mdr_usage_report import MdrUsageReport as MdrUsageReport
from .pagination_meta_reporting import PaginationMetaReporting as PaginationMetaReporting
from .mdr_usage_report_list_params import MdrUsageReportListParams as MdrUsageReportListParams
from .mdr_usage_report_create_params import MdrUsageReportCreateParams as MdrUsageReportCreateParams
from .mdr_usage_report_list_response import MdrUsageReportListResponse as MdrUsageReportListResponse
from .mdr_usage_report_create_response import MdrUsageReportCreateResponse as MdrUsageReportCreateResponse
from .mdr_usage_report_delete_response import MdrUsageReportDeleteResponse as MdrUsageReportDeleteResponse
from .cdr_usage_report_fetch_sync_params import CdrUsageReportFetchSyncParams as CdrUsageReportFetchSyncParams
from .mdr_usage_report_fetch_sync_params import MdrUsageReportFetchSyncParams as MdrUsageReportFetchSyncParams
from .mdr_usage_report_retrieve_response import MdrUsageReportRetrieveResponse as MdrUsageReportRetrieveResponse
from .cdr_usage_report_fetch_sync_response import CdrUsageReportFetchSyncResponse as CdrUsageReportFetchSyncResponse
from .mdr_usage_report_fetch_sync_response import MdrUsageReportFetchSyncResponse as MdrUsageReportFetchSyncResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Literal, Required, TypeAlias, TypedDict

from ..._types import SequenceNotStr
from ..calls.aws_voice_settings_param import AwsVoiceSettingsParam
from ..calls.telnyx_voice_settings_param import TelnyxVoiceSettingsParam
from ..calls.eleven_labs_voice_settings_param import ElevenLabsVoiceSettingsParam

__all__ = ["ActionSpeakParams", "VoiceSettings"]


class ActionSpeakParams(TypedDict, total=False):
    payload: Required[str]
    """The text or SSML to be converted into speech. There is a 3,000 character limit."""

    voice: Required[str]
    """Specifies the voice used in speech synthesis.

    - Define voices using the format `<Provider>.<Model>.<VoiceId>`. Specifying only
      the provider will give default values for voice_id and model_id.

      **Supported Providers:**

    - **AWS:** Use `AWS.Polly.<VoiceId>` (e.g., `AWS.Polly.Joanna`). For neural
      voices, which provide more realistic, human-like speech, append `-Neural` to
      the `VoiceId` (e.g., `AWS.Polly.Joanna-Neural`). Check the
      [available voices](https://docs.aws.amazon.com/polly/latest/dg/available-voices.html)
      for compatibility.
    - **Azure:** Use `Azure.<VoiceId>. (e.g. Azure.en-CA-ClaraNeural,
      Azure.en-CA-LiamNeural, Azure.en-US-BrianMultilingualNeural,
      Azure.en-US-Ava:DragonHDLatestNeural. For a complete list of voices, go to
      [Azure Voice Gallery](https://speech.microsoft.com/portal/voicegallery).)
    - **ElevenLabs:** Use `ElevenLabs.<ModelId>.<VoiceId>` (e.g.,
      `ElevenLabs.eleven_multilingual_v2.21m00Tcm4TlvDq8ikWAM`). The `ModelId` part
      is optional. To use ElevenLabs, you must provide your ElevenLabs API key as an
      integration identifier secret in
      `"voice_settings": {"api_key_ref": "<secret_identifier>"}`. See
      [integration secrets documentation](https://developers.telnyx.com/api/secrets-manager/integration-secrets/create-integration-secret)
      for details. Check
      [available voices](https://elevenlabs.io/docs/api-reference/get-voices).
    - **Telnyx:** Use `Telnyx.<model_id>.<voice_id>`

    For service_level basic, you may define the gender of the speaker (male or
    female).
    """

    call_control_ids: SequenceNotStr[str]
    """Call Control IDs of participants who will hear the spoken text.

    When empty all participants will hear the spoken text.
    """

    command_id: str
    """Use this field to avoid execution of duplicate commands.

    Telnyx will ignore subsequent commands with the same `command_id` as one that
    has already been executed.
    """

    language: Literal[
        "arb",
        "cmn-CN",
        "cy-GB",
        "da-DK",
        "de-DE",
        "en-AU",
        "en-GB",
        "en-GB-WLS",
        "en-IN",
        "en-US",
        "es-ES",
        "es-MX",
        "es-US",
        "fr-CA",
        "fr-FR",
        "hi-IN",
        "is-IS",
        "it-IT",
        "ja-JP",
        "ko-KR",
        "nb-NO",
        "nl-NL",
        "pl-PL",
        "pt-BR",
        "pt-PT",
        "ro-RO",
        "ru-RU",
        "sv-SE",
        "tr-TR",
    ]
    """The language you want spoken.

    This parameter is ignored when a `Polly.*` voice is specified.
    """

    payload_type: Literal["text", "ssml"]
    """The type of the provided payload.

    The payload can either be plain text, or Speech Synthesis Markup Language
    (SSML).
    """

    voice_settings: VoiceSettings
    """The settings associated with the voice selected"""


VoiceSettings: TypeAlias = Union[ElevenLabsVoiceSettingsParam, TelnyxVoiceSettingsParam, AwsVoiceSettingsParam]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Union, Optional
from typing_extensions import Literal, TypeAlias

from ..._models import BaseModel
from .hangup_tool import HangupTool
from .webhook_tool import WebhookTool
from .transfer_tool import TransferTool
from .retrieval_tool import RetrievalTool

__all__ = [
    "AssistantTool",
    "HandoffTool",
    "HandoffToolHandoff",
    "HandoffToolHandoffAIAssistant",
    "SipReferTool",
    "SipReferToolRefer",
    "SipReferToolReferTarget",
    "SipReferToolReferCustomHeader",
    "SipReferToolReferSipHeader",
    "DtmfTool",
]


class HandoffToolHandoffAIAssistant(BaseModel):
    id: str
    """The ID of the assistant to hand off to."""

    name: str
    """Helpful name for giving context on when to handoff to the assistant."""


class HandoffToolHandoff(BaseModel):
    ai_assistants: List[HandoffToolHandoffAIAssistant]
    """List of possible assistants that can receive a handoff."""

    voice_mode: Optional[Literal["unified", "distinct"]] = None
    """
    With the unified voice mode all assistants share the same voice, making the
    handoff transparent to the user. With the distinct voice mode all assistants
    retain their voice configuration, providing the experience of a conference call
    with a team of assistants.
    """


class HandoffTool(BaseModel):
    handoff: HandoffToolHandoff

    type: Literal["handoff"]


class SipReferToolReferTarget(BaseModel):
    name: str
    """The name of the target."""

    sip_address: str
    """The SIP URI to which the call will be referred."""

    sip_auth_password: Optional[str] = None
    """SIP Authentication password used for SIP challenges."""

    sip_auth_username: Optional[str] = None
    """SIP Authentication username used for SIP challenges."""


class SipReferToolReferCustomHeader(BaseModel):
    name: Optional[str] = None

    value: Optional[str] = None
    """The value of the header.

    Note that we support mustache templating for the value. For example you can use
    `{{#integration_secret}}test-secret{{/integration_secret}}` to pass the value of
    the integration secret.
    """


class SipReferToolReferSipHeader(BaseModel):
    name: Optional[Literal["User-to-User", "Diversion"]] = None

    value: Optional[str] = None
    """The value of the header.

    Note that we support mustache templating for the value. For example you can use
    `{{#integration_secret}}test-secret{{/integration_secret}}` to pass the value of
    the integration secret.
    """


class SipReferToolRefer(BaseModel):
    targets: List[SipReferToolReferTarget]
    """The different possible targets of the SIP refer.

    The assistant will be able to choose one of the targets to refer the call to.
    """

    custom_headers: Optional[List[SipReferToolReferCustomHeader]] = None
    """Custom headers to be added to the SIP REFER."""

    sip_headers: Optional[List[SipReferToolReferSipHeader]] = None
    """SIP headers to be added to the SIP REFER.

    Currently only User-to-User and Diversion headers are supported.
    """


class SipReferTool(BaseModel):
    refer: SipReferToolRefer

    type: Literal["refer"]


class DtmfTool(BaseModel):
    send_dtmf: Dict[str, object]

    type: Literal["send_dtmf"]


AssistantTool: TypeAlias = Union[
    WebhookTool, RetrievalTool, HandoffTool, HangupTool, TransferTool, SipReferTool, DtmfTool
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["SoundModificationsParam"]


class SoundModificationsParam(TypedDict, total=False):
    octaves: float
    """Adjust the pitch in octaves, values should be between -1 and 1, default 0"""

    pitch: float
    """Set the pitch directly, value should be > 0, default 1 (lower = lower tone)"""

    semitone: float
    """Adjust the pitch in semitones, values should be between -14 and 14, default 0"""

    track: str
    """The track to which the sound modifications will be applied.

    Accepted values are `inbound` or `outbound`
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from .._models import BaseModel
from .pagination_meta import PaginationMeta

__all__ = ["NotificationEventListResponse", "Data"]


class Data(BaseModel):
    id: Optional[str] = None
    """A UUID."""

    created_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was created."""

    enabled: Optional[bool] = None

    name: Optional[str] = None
    """A human readable name."""

    notification_category: Optional[str] = None

    updated_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was updated."""


class NotificationEventListResponse(BaseModel):
    data: Optional[List[Data]] = None

    meta: Optional[PaginationMeta] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel
from .oauth_grant import OAuthGrant
from .pagination_meta_oauth import PaginationMetaOAuth

__all__ = ["OAuthGrantListResponse"]


class OAuthGrantListResponse(BaseModel):
    data: Optional[List[OAuthGrant]] = None

    meta: Optional[PaginationMetaOAuth] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..types import messsage_rcs_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.messsage_rcs_response import MesssageRcsResponse
from ..types.rcs_agent_message_param import RcsAgentMessageParam

__all__ = ["MesssagesResource", "AsyncMesssagesResource"]


class MesssagesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> MesssagesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return MesssagesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> MesssagesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return MesssagesResourceWithStreamingResponse(self)

    def rcs(
        self,
        *,
        agent_id: str,
        agent_message: RcsAgentMessageParam,
        messaging_profile_id: str,
        to: str,
        mms_fallback: messsage_rcs_params.MmsFallback | Omit = omit,
        sms_fallback: messsage_rcs_params.SMSFallback | Omit = omit,
        type: Literal["RCS"] | Omit = omit,
        webhook_url: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MesssageRcsResponse:
        """
        Send an RCS message

        Args:
          agent_id: RCS Agent ID

          messaging_profile_id: A valid messaging profile ID

          to: Phone number in +E.164 format

          type: Message type - must be set to "RCS"

          webhook_url: The URL where webhooks related to this message will be sent.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/messsages/rcs",
            body=maybe_transform(
                {
                    "agent_id": agent_id,
                    "agent_message": agent_message,
                    "messaging_profile_id": messaging_profile_id,
                    "to": to,
                    "mms_fallback": mms_fallback,
                    "sms_fallback": sms_fallback,
                    "type": type,
                    "webhook_url": webhook_url,
                },
                messsage_rcs_params.MesssageRcsParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MesssageRcsResponse,
        )


class AsyncMesssagesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncMesssagesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncMesssagesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncMesssagesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncMesssagesResourceWithStreamingResponse(self)

    async def rcs(
        self,
        *,
        agent_id: str,
        agent_message: RcsAgentMessageParam,
        messaging_profile_id: str,
        to: str,
        mms_fallback: messsage_rcs_params.MmsFallback | Omit = omit,
        sms_fallback: messsage_rcs_params.SMSFallback | Omit = omit,
        type: Literal["RCS"] | Omit = omit,
        webhook_url: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MesssageRcsResponse:
        """
        Send an RCS message

        Args:
          agent_id: RCS Agent ID

          messaging_profile_id: A valid messaging profile ID

          to: Phone number in +E.164 format

          type: Message type - must be set to "RCS"

          webhook_url: The URL where webhooks related to this message will be sent.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/messsages/rcs",
            body=await async_maybe_transform(
                {
                    "agent_id": agent_id,
                    "agent_message": agent_message,
                    "messaging_profile_id": messaging_profile_id,
                    "to": to,
                    "mms_fallback": mms_fallback,
                    "sms_fallback": sms_fallback,
                    "type": type,
                    "webhook_url": webhook_url,
                },
                messsage_rcs_params.MesssageRcsParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MesssageRcsResponse,
        )


class MesssagesResourceWithRawResponse:
    def __init__(self, messsages: MesssagesResource) -> None:
        self._messsages = messsages

        self.rcs = to_raw_response_wrapper(
            messsages.rcs,
        )


class AsyncMesssagesResourceWithRawResponse:
    def __init__(self, messsages: AsyncMesssagesResource) -> None:
        self._messsages = messsages

        self.rcs = async_to_raw_response_wrapper(
            messsages.rcs,
        )


class MesssagesResourceWithStreamingResponse:
    def __init__(self, messsages: MesssagesResource) -> None:
        self._messsages = messsages

        self.rcs = to_streamed_response_wrapper(
            messsages.rcs,
        )


class AsyncMesssagesResourceWithStreamingResponse:
    def __init__(self, messsages: AsyncMesssagesResource) -> None:
        self._messsages = messsages

        self.rcs = async_to_streamed_response_wrapper(
            messsages.rcs,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import date

from .._models import BaseModel

__all__ = ["InvoiceListResponse", "Data", "Meta"]


class Data(BaseModel):
    file_id: Optional[str] = None

    invoice_id: Optional[str] = None

    paid: Optional[bool] = None

    period_end: Optional[date] = None

    period_start: Optional[date] = None

    url: Optional[str] = None


class Meta(BaseModel):
    page_number: Optional[int] = None

    page_size: Optional[int] = None

    total_pages: Optional[int] = None

    total_results: Optional[int] = None


class InvoiceListResponse(BaseModel):
    data: Optional[List[Data]] = None

    meta: Optional[Meta] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["URLShortenerSettings"]


class URLShortenerSettings(BaseModel):
    domain: str
    """One of the domains provided by the Telnyx URL shortener service."""

    prefix: Optional[str] = None
    """
    Optional prefix that can be used to identify your brand, and will appear in the
    Telnyx generated URLs after the domain name.
    """

    replace_blacklist_only: Optional[bool] = None
    """
    Use the link replacement tool only for links that are specifically blacklisted
    by Telnyx.
    """

    send_webhooks: Optional[bool] = None
    """Receive webhooks for when your replaced links are clicked.

    Webhooks are sent to the webhooks on the messaging profile.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = [
    "NotificationSettingListParams",
    "Filter",
    "FilterAssociatedRecordType",
    "FilterChannelTypeID",
    "FilterNotificationChannel",
    "FilterNotificationEventConditionID",
    "FilterNotificationProfileID",
    "FilterStatus",
    "Page",
]


class NotificationSettingListParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter (deepObject style).

    Originally: filter[associated_record_type][eq], filter[channel_type_id][eq],
    filter[notification_profile_id][eq], filter[notification_channel][eq],
    filter[notification_event_condition_id][eq], filter[status][eq]
    """

    page: Page
    """Consolidated page parameter (deepObject style).

    Originally: page[number], page[size]
    """


class FilterAssociatedRecordType(TypedDict, total=False):
    eq: Literal["account", "phone_number"]
    """Filter by the associated record type"""


class FilterChannelTypeID(TypedDict, total=False):
    eq: Literal["webhook", "sms", "email", "voice"]
    """Filter by the id of a channel type"""


class FilterNotificationChannel(TypedDict, total=False):
    eq: str
    """Filter by the id of a notification channel"""


class FilterNotificationEventConditionID(TypedDict, total=False):
    eq: str
    """Filter by the id of a notification channel"""


class FilterNotificationProfileID(TypedDict, total=False):
    eq: str
    """Filter by the id of a notification profile"""


class FilterStatus(TypedDict, total=False):
    eq: Literal[
        "enabled",
        "enable-received",
        "enable-pending",
        "enable-submtited",
        "delete-received",
        "delete-pending",
        "delete-submitted",
        "deleted",
    ]
    """The status of a notification setting"""


class Filter(TypedDict, total=False):
    associated_record_type: FilterAssociatedRecordType

    channel_type_id: FilterChannelTypeID

    notification_channel: FilterNotificationChannel

    notification_event_condition_id: FilterNotificationEventConditionID

    notification_profile_id: FilterNotificationProfileID

    status: FilterStatus


class Page(TypedDict, total=False):
    number: int
    """The page number to load"""

    size: int
    """The size of the page"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .number_reservation import NumberReservation

__all__ = ["NumberReservationCreateResponse"]


class NumberReservationCreateResponse(BaseModel):
    data: Optional[NumberReservation] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["TranscriptionWebhookEvent", "Data", "DataPayload", "DataPayloadTranscriptionData"]


class DataPayloadTranscriptionData(BaseModel):
    confidence: Optional[float] = None
    """Speech recognition confidence level."""

    is_final: Optional[bool] = None
    """When false, it means that this is an interim result."""

    transcript: Optional[str] = None
    """Recognized text."""

    transcription_track: Optional[Literal["inbound", "outbound"]] = None
    """Indicates which leg of the call has been transcribed.

    This is only available when `transcription_engine` is set to `B`.
    """


class DataPayload(BaseModel):
    call_control_id: Optional[str] = None
    """Unique identifier and token for controlling the call."""

    call_leg_id: Optional[str] = None
    """ID that is unique to the call and can be used to correlate webhook events."""

    call_session_id: Optional[str] = None
    """
    ID that is unique to the call session and can be used to correlate webhook
    events. Call session is a group of related call legs that logically belong to
    the same phone call, e.g. an inbound and outbound leg of a transferred call.
    """

    client_state: Optional[str] = None
    """Use this field to add state to every subsequent webhook.

    It must be a valid Base-64 encoded string.
    """

    connection_id: Optional[str] = None
    """Call Control App ID (formerly Telnyx connection ID) used in the call."""

    transcription_data: Optional[DataPayloadTranscriptionData] = None


class Data(BaseModel):
    id: Optional[str] = None
    """Identifies the type of resource."""

    event_type: Optional[Literal["call.transcription"]] = None
    """The type of event being delivered."""

    occurred_at: Optional[datetime] = None
    """ISO 8601 datetime of when the event occurred."""

    payload: Optional[DataPayload] = None

    record_type: Optional[Literal["event"]] = None
    """Identifies the type of the resource."""


class TranscriptionWebhookEvent(BaseModel):
    data: Optional[Data] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = [
    "GlobalIPLatencyRetrieveResponse",
    "Data",
    "DataGlobalIP",
    "DataMeanLatency",
    "DataPercentileLatency",
    "DataPercentileLatency_0",
    "DataPercentileLatency_100",
    "DataPercentileLatency_25",
    "DataPercentileLatency_50",
    "DataPercentileLatency_75",
    "DataPercentileLatency_90",
    "DataPercentileLatency_99",
    "DataProberLocation",
]


class DataGlobalIP(BaseModel):
    id: Optional[str] = None
    """Global IP ID."""

    ip_address: Optional[str] = None
    """The Global IP address."""


class DataMeanLatency(BaseModel):
    amount: Optional[float] = None
    """The average latency."""

    unit: Optional[str] = None
    """The unit of the average latency."""


class DataPercentileLatency_0(BaseModel):
    amount: Optional[float] = None
    """The minimum latency."""

    unit: Optional[str] = None
    """The unit of the minimum latency."""


class DataPercentileLatency_100(BaseModel):
    amount: Optional[float] = None
    """The maximum latency."""

    unit: Optional[str] = None
    """The unit of the maximum latency."""


class DataPercentileLatency_25(BaseModel):
    amount: Optional[float] = None
    """The 25th percentile latency."""

    unit: Optional[str] = None
    """The unit of the 25th percentile latency."""


class DataPercentileLatency_50(BaseModel):
    amount: Optional[float] = None
    """The 50th percentile latency."""

    unit: Optional[str] = None
    """The unit of the 50th percentile latency."""


class DataPercentileLatency_75(BaseModel):
    amount: Optional[float] = None
    """The 75th percentile latency."""

    unit: Optional[str] = None
    """The unit of the 75th percentile latency."""


class DataPercentileLatency_90(BaseModel):
    amount: Optional[float] = None
    """The 90th percentile latency."""

    unit: Optional[str] = None
    """The unit of the 90th percentile latency."""


class DataPercentileLatency_99(BaseModel):
    amount: Optional[float] = None
    """The 99th percentile latency."""

    unit: Optional[str] = None
    """The unit of the 99th percentile latency."""


class DataPercentileLatency(BaseModel):
    p0: Optional[DataPercentileLatency_0] = FieldInfo(alias="0", default=None)

    p100: Optional[DataPercentileLatency_100] = FieldInfo(alias="100", default=None)

    p25: Optional[DataPercentileLatency_25] = FieldInfo(alias="25", default=None)

    p50: Optional[DataPercentileLatency_50] = FieldInfo(alias="50", default=None)

    p75: Optional[DataPercentileLatency_75] = FieldInfo(alias="75", default=None)

    p90: Optional[DataPercentileLatency_90] = FieldInfo(alias="90", default=None)

    p99: Optional[DataPercentileLatency_99] = FieldInfo(alias="99", default=None)


class DataProberLocation(BaseModel):
    id: Optional[str] = None
    """Location ID."""

    lat: Optional[float] = None
    """Latitude."""

    lon: Optional[float] = None
    """Longitude."""

    name: Optional[str] = None
    """Location name."""


class Data(BaseModel):
    global_ip: Optional[DataGlobalIP] = None

    mean_latency: Optional[DataMeanLatency] = None

    percentile_latency: Optional[DataPercentileLatency] = None

    prober_location: Optional[DataProberLocation] = None

    timestamp: Optional[datetime] = None
    """The timestamp of the metric."""


class GlobalIPLatencyRetrieveResponse(BaseModel):
    data: Optional[List[Data]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ..._models import BaseModel
from .auto_resp_config import AutoRespConfig

__all__ = ["AutoRespConfigResponse"]


class AutoRespConfigResponse(BaseModel):
    data: AutoRespConfig



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from datetime import datetime

from ..._models import BaseModel
from ..task_status import TaskStatus
from .assistants.tests.test_suites.meta import Meta

__all__ = ["ClusterListResponse", "Data"]


class Data(BaseModel):
    bucket: str

    created_at: datetime

    finished_at: datetime

    min_cluster_size: int

    min_subcluster_size: int

    status: TaskStatus

    task_id: str


class ClusterListResponse(BaseModel):
    data: List[Data]

    meta: Meta



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

from ...._types import FileTypes

__all__ = ["SslCertificateCreateParams"]


class SslCertificateCreateParams(TypedDict, total=False):
    certificate: FileTypes
    """The SSL certificate file"""

    private_key: FileTypes
    """The private key file"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types import (
    MessagingProfileListResponse,
    MessagingProfileCreateResponse,
    MessagingProfileDeleteResponse,
    MessagingProfileUpdateResponse,
    MessagingProfileRetrieveResponse,
    MessagingProfileListShortCodesResponse,
    MessagingProfileListPhoneNumbersResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestMessagingProfiles:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: Telnyx) -> None:
        messaging_profile = client.messaging_profiles.create(
            name="My name",
            whitelisted_destinations=["US"],
        )
        assert_matches_type(MessagingProfileCreateResponse, messaging_profile, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Telnyx) -> None:
        messaging_profile = client.messaging_profiles.create(
            name="My name",
            whitelisted_destinations=["US"],
            alpha_sender="sqF",
            daily_spend_limit="269125115713",
            daily_spend_limit_enabled=True,
            enabled=True,
            mms_fall_back_to_sms=True,
            mms_transcoding=True,
            mobile_only=True,
            number_pool_settings={
                "long_code_weight": 1,
                "skip_unhealthy": True,
                "toll_free_weight": 10,
                "geomatch": False,
                "sticky_sender": False,
            },
            url_shortener_settings={
                "domain": "example.ex",
                "prefix": "",
                "replace_blacklist_only": True,
                "send_webhooks": False,
            },
            webhook_api_version="2",
            webhook_failover_url="https://backup.example.com/hooks",
            webhook_url="https://www.example.com/hooks",
        )
        assert_matches_type(MessagingProfileCreateResponse, messaging_profile, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Telnyx) -> None:
        response = client.messaging_profiles.with_raw_response.create(
            name="My name",
            whitelisted_destinations=["US"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        messaging_profile = response.parse()
        assert_matches_type(MessagingProfileCreateResponse, messaging_profile, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Telnyx) -> None:
        with client.messaging_profiles.with_streaming_response.create(
            name="My name",
            whitelisted_destinations=["US"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            messaging_profile = response.parse()
            assert_matches_type(MessagingProfileCreateResponse, messaging_profile, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        messaging_profile = client.messaging_profiles.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(MessagingProfileRetrieveResponse, messaging_profile, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.messaging_profiles.with_raw_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        messaging_profile = response.parse()
        assert_matches_type(MessagingProfileRetrieveResponse, messaging_profile, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.messaging_profiles.with_streaming_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            messaging_profile = response.parse()
            assert_matches_type(MessagingProfileRetrieveResponse, messaging_profile, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.messaging_profiles.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update(self, client: Telnyx) -> None:
        messaging_profile = client.messaging_profiles.update(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(MessagingProfileUpdateResponse, messaging_profile, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: Telnyx) -> None:
        messaging_profile = client.messaging_profiles.update(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            alpha_sender="sqF",
            daily_spend_limit="269125115713",
            daily_spend_limit_enabled=True,
            enabled=True,
            mms_fall_back_to_sms=True,
            mms_transcoding=True,
            mobile_only=True,
            name="Updated Profile for Messages",
            number_pool_settings={
                "long_code_weight": 2,
                "skip_unhealthy": False,
                "toll_free_weight": 10,
                "geomatch": False,
                "sticky_sender": True,
            },
            url_shortener_settings={
                "domain": "example.ex",
                "prefix": "cmpny",
                "replace_blacklist_only": True,
                "send_webhooks": False,
            },
            v1_secret="rP1VamejkU2v0qIUxntqLW2c",
            webhook_api_version="2",
            webhook_failover_url="https://backup.example.com/hooks",
            webhook_url="https://www.example.com/hooks",
            whitelisted_destinations=["US"],
        )
        assert_matches_type(MessagingProfileUpdateResponse, messaging_profile, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Telnyx) -> None:
        response = client.messaging_profiles.with_raw_response.update(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        messaging_profile = response.parse()
        assert_matches_type(MessagingProfileUpdateResponse, messaging_profile, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Telnyx) -> None:
        with client.messaging_profiles.with_streaming_response.update(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            messaging_profile = response.parse()
            assert_matches_type(MessagingProfileUpdateResponse, messaging_profile, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.messaging_profiles.with_raw_response.update(
                id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        messaging_profile = client.messaging_profiles.list()
        assert_matches_type(MessagingProfileListResponse, messaging_profile, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        messaging_profile = client.messaging_profiles.list(
            filter={"name": "name"},
            page={
                "number": 1,
                "size": 1,
            },
        )
        assert_matches_type(MessagingProfileListResponse, messaging_profile, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.messaging_profiles.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        messaging_profile = response.parse()
        assert_matches_type(MessagingProfileListResponse, messaging_profile, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.messaging_profiles.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            messaging_profile = response.parse()
            assert_matches_type(MessagingProfileListResponse, messaging_profile, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_delete(self, client: Telnyx) -> None:
        messaging_profile = client.messaging_profiles.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(MessagingProfileDeleteResponse, messaging_profile, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Telnyx) -> None:
        response = client.messaging_profiles.with_raw_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        messaging_profile = response.parse()
        assert_matches_type(MessagingProfileDeleteResponse, messaging_profile, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Telnyx) -> None:
        with client.messaging_profiles.with_streaming_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            messaging_profile = response.parse()
            assert_matches_type(MessagingProfileDeleteResponse, messaging_profile, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.messaging_profiles.with_raw_response.delete(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_phone_numbers(self, client: Telnyx) -> None:
        messaging_profile = client.messaging_profiles.list_phone_numbers(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(MessagingProfileListPhoneNumbersResponse, messaging_profile, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_phone_numbers_with_all_params(self, client: Telnyx) -> None:
        messaging_profile = client.messaging_profiles.list_phone_numbers(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            page={
                "number": 1,
                "size": 1,
            },
        )
        assert_matches_type(MessagingProfileListPhoneNumbersResponse, messaging_profile, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list_phone_numbers(self, client: Telnyx) -> None:
        response = client.messaging_profiles.with_raw_response.list_phone_numbers(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        messaging_profile = response.parse()
        assert_matches_type(MessagingProfileListPhoneNumbersResponse, messaging_profile, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list_phone_numbers(self, client: Telnyx) -> None:
        with client.messaging_profiles.with_streaming_response.list_phone_numbers(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            messaging_profile = response.parse()
            assert_matches_type(MessagingProfileListPhoneNumbersResponse, messaging_profile, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_list_phone_numbers(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.messaging_profiles.with_raw_response.list_phone_numbers(
                id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_short_codes(self, client: Telnyx) -> None:
        messaging_profile = client.messaging_profiles.list_short_codes(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(MessagingProfileListShortCodesResponse, messaging_profile, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_short_codes_with_all_params(self, client: Telnyx) -> None:
        messaging_profile = client.messaging_profiles.list_short_codes(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            page={
                "number": 1,
                "size": 1,
            },
        )
        assert_matches_type(MessagingProfileListShortCodesResponse, messaging_profile, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list_short_codes(self, client: Telnyx) -> None:
        response = client.messaging_profiles.with_raw_response.list_short_codes(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        messaging_profile = response.parse()
        assert_matches_type(MessagingProfileListShortCodesResponse, messaging_profile, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list_short_codes(self, client: Telnyx) -> None:
        with client.messaging_profiles.with_streaming_response.list_short_codes(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            messaging_profile = response.parse()
            assert_matches_type(MessagingProfileListShortCodesResponse, messaging_profile, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_list_short_codes(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.messaging_profiles.with_raw_response.list_short_codes(
                id="",
            )


class TestAsyncMessagingProfiles:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncTelnyx) -> None:
        messaging_profile = await async_client.messaging_profiles.create(
            name="My name",
            whitelisted_destinations=["US"],
        )
        assert_matches_type(MessagingProfileCreateResponse, messaging_profile, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncTelnyx) -> None:
        messaging_profile = await async_client.messaging_profiles.create(
            name="My name",
            whitelisted_destinations=["US"],
            alpha_sender="sqF",
            daily_spend_limit="269125115713",
            daily_spend_limit_enabled=True,
            enabled=True,
            mms_fall_back_to_sms=True,
            mms_transcoding=True,
            mobile_only=True,
            number_pool_settings={
                "long_code_weight": 1,
                "skip_unhealthy": True,
                "toll_free_weight": 10,
                "geomatch": False,
                "sticky_sender": False,
            },
            url_shortener_settings={
                "domain": "example.ex",
                "prefix": "",
                "replace_blacklist_only": True,
                "send_webhooks": False,
            },
            webhook_api_version="2",
            webhook_failover_url="https://backup.example.com/hooks",
            webhook_url="https://www.example.com/hooks",
        )
        assert_matches_type(MessagingProfileCreateResponse, messaging_profile, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.messaging_profiles.with_raw_response.create(
            name="My name",
            whitelisted_destinations=["US"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        messaging_profile = await response.parse()
        assert_matches_type(MessagingProfileCreateResponse, messaging_profile, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncTelnyx) -> None:
        async with async_client.messaging_profiles.with_streaming_response.create(
            name="My name",
            whitelisted_destinations=["US"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            messaging_profile = await response.parse()
            assert_matches_type(MessagingProfileCreateResponse, messaging_profile, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        messaging_profile = await async_client.messaging_profiles.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(MessagingProfileRetrieveResponse, messaging_profile, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.messaging_profiles.with_raw_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        messaging_profile = await response.parse()
        assert_matches_type(MessagingProfileRetrieveResponse, messaging_profile, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.messaging_profiles.with_streaming_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            messaging_profile = await response.parse()
            assert_matches_type(MessagingProfileRetrieveResponse, messaging_profile, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.messaging_profiles.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncTelnyx) -> None:
        messaging_profile = await async_client.messaging_profiles.update(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(MessagingProfileUpdateResponse, messaging_profile, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncTelnyx) -> None:
        messaging_profile = await async_client.messaging_profiles.update(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            alpha_sender="sqF",
            daily_spend_limit="269125115713",
            daily_spend_limit_enabled=True,
            enabled=True,
            mms_fall_back_to_sms=True,
            mms_transcoding=True,
            mobile_only=True,
            name="Updated Profile for Messages",
            number_pool_settings={
                "long_code_weight": 2,
                "skip_unhealthy": False,
                "toll_free_weight": 10,
                "geomatch": False,
                "sticky_sender": True,
            },
            url_shortener_settings={
                "domain": "example.ex",
                "prefix": "cmpny",
                "replace_blacklist_only": True,
                "send_webhooks": False,
            },
            v1_secret="rP1VamejkU2v0qIUxntqLW2c",
            webhook_api_version="2",
            webhook_failover_url="https://backup.example.com/hooks",
            webhook_url="https://www.example.com/hooks",
            whitelisted_destinations=["US"],
        )
        assert_matches_type(MessagingProfileUpdateResponse, messaging_profile, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.messaging_profiles.with_raw_response.update(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        messaging_profile = await response.parse()
        assert_matches_type(MessagingProfileUpdateResponse, messaging_profile, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncTelnyx) -> None:
        async with async_client.messaging_profiles.with_streaming_response.update(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            messaging_profile = await response.parse()
            assert_matches_type(MessagingProfileUpdateResponse, messaging_profile, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.messaging_profiles.with_raw_response.update(
                id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        messaging_profile = await async_client.messaging_profiles.list()
        assert_matches_type(MessagingProfileListResponse, messaging_profile, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        messaging_profile = await async_client.messaging_profiles.list(
            filter={"name": "name"},
            page={
                "number": 1,
                "size": 1,
            },
        )
        assert_matches_type(MessagingProfileListResponse, messaging_profile, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.messaging_profiles.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        messaging_profile = await response.parse()
        assert_matches_type(MessagingProfileListResponse, messaging_profile, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.messaging_profiles.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            messaging_profile = await response.parse()
            assert_matches_type(MessagingProfileListResponse, messaging_profile, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncTelnyx) -> None:
        messaging_profile = await async_client.messaging_profiles.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(MessagingProfileDeleteResponse, messaging_profile, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.messaging_profiles.with_raw_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        messaging_profile = await response.parse()
        assert_matches_type(MessagingProfileDeleteResponse, messaging_profile, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncTelnyx) -> None:
        async with async_client.messaging_profiles.with_streaming_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            messaging_profile = await response.parse()
            assert_matches_type(MessagingProfileDeleteResponse, messaging_profile, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.messaging_profiles.with_raw_response.delete(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_phone_numbers(self, async_client: AsyncTelnyx) -> None:
        messaging_profile = await async_client.messaging_profiles.list_phone_numbers(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(MessagingProfileListPhoneNumbersResponse, messaging_profile, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_phone_numbers_with_all_params(self, async_client: AsyncTelnyx) -> None:
        messaging_profile = await async_client.messaging_profiles.list_phone_numbers(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            page={
                "number": 1,
                "size": 1,
            },
        )
        assert_matches_type(MessagingProfileListPhoneNumbersResponse, messaging_profile, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list_phone_numbers(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.messaging_profiles.with_raw_response.list_phone_numbers(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        messaging_profile = await response.parse()
        assert_matches_type(MessagingProfileListPhoneNumbersResponse, messaging_profile, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list_phone_numbers(self, async_client: AsyncTelnyx) -> None:
        async with async_client.messaging_profiles.with_streaming_response.list_phone_numbers(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            messaging_profile = await response.parse()
            assert_matches_type(MessagingProfileListPhoneNumbersResponse, messaging_profile, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_list_phone_numbers(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.messaging_profiles.with_raw_response.list_phone_numbers(
                id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_short_codes(self, async_client: AsyncTelnyx) -> None:
        messaging_profile = await async_client.messaging_profiles.list_short_codes(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(MessagingProfileListShortCodesResponse, messaging_profile, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_short_codes_with_all_params(self, async_client: AsyncTelnyx) -> None:
        messaging_profile = await async_client.messaging_profiles.list_short_codes(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            page={
                "number": 1,
                "size": 1,
            },
        )
        assert_matches_type(MessagingProfileListShortCodesResponse, messaging_profile, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list_short_codes(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.messaging_profiles.with_raw_response.list_short_codes(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        messaging_profile = await response.parse()
        assert_matches_type(MessagingProfileListShortCodesResponse, messaging_profile, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list_short_codes(self, async_client: AsyncTelnyx) -> None:
        async with async_client.messaging_profiles.with_streaming_response.list_short_codes(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            messaging_profile = await response.parse()
            assert_matches_type(MessagingProfileListShortCodesResponse, messaging_profile, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_list_short_codes(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.messaging_profiles.with_raw_response.list_short_codes(
                id="",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Required, Annotated, TypedDict

from .._types import SequenceNotStr
from .._utils import PropertyInfo

__all__ = ["CampaignBuilderCreateParams"]


class CampaignBuilderCreateParams(TypedDict, total=False):
    brand_id: Required[Annotated[str, PropertyInfo(alias="brandId")]]
    """Alphanumeric identifier of the brand associated with this campaign."""

    description: Required[str]
    """Summary description of this campaign."""

    usecase: Required[str]
    """Campaign usecase.

    Must be of defined valid types. Use `/registry/enum/usecase` operation to
    retrieve usecases available for given brand.
    """

    age_gated: Annotated[bool, PropertyInfo(alias="ageGated")]
    """Age gated message content in campaign."""

    auto_renewal: Annotated[bool, PropertyInfo(alias="autoRenewal")]
    """Campaign subscription auto-renewal option.

    If set to true, then campaign will automatically renewal at end of billing
    cycle.
    """

    direct_lending: Annotated[bool, PropertyInfo(alias="directLending")]
    """Direct lending or loan arrangement"""

    embedded_link: Annotated[bool, PropertyInfo(alias="embeddedLink")]
    """Does message generated by the campaign include URL link in SMS?"""

    embedded_link_sample: Annotated[str, PropertyInfo(alias="embeddedLinkSample")]
    """Sample of an embedded link that will be sent to subscribers."""

    embedded_phone: Annotated[bool, PropertyInfo(alias="embeddedPhone")]
    """Does message generated by the campaign include phone number in SMS?"""

    help_keywords: Annotated[str, PropertyInfo(alias="helpKeywords")]
    """Subscriber help keywords. Multiple keywords are comma separated without space."""

    help_message: Annotated[str, PropertyInfo(alias="helpMessage")]
    """Help message of the campaign."""

    message_flow: Annotated[str, PropertyInfo(alias="messageFlow")]
    """Message flow description."""

    mno_ids: Annotated[Iterable[int], PropertyInfo(alias="mnoIds")]
    """Submit campaign to given list of MNOs by MNO's network ID.

    Default is all MNOs if no value provided.
    """

    number_pool: Annotated[bool, PropertyInfo(alias="numberPool")]
    """Does campaign utilize pool of phone numbers?"""

    optin_keywords: Annotated[str, PropertyInfo(alias="optinKeywords")]
    """Subscriber opt-in keywords.

    Multiple keywords are comma separated without space.
    """

    optin_message: Annotated[str, PropertyInfo(alias="optinMessage")]
    """Subscriber opt-in message."""

    optout_keywords: Annotated[str, PropertyInfo(alias="optoutKeywords")]
    """Subscriber opt-out keywords.

    Multiple keywords are comma separated without space.
    """

    optout_message: Annotated[str, PropertyInfo(alias="optoutMessage")]
    """Subscriber opt-out message."""

    privacy_policy_link: Annotated[str, PropertyInfo(alias="privacyPolicyLink")]
    """Link to the campaign's privacy policy."""

    reference_id: Annotated[str, PropertyInfo(alias="referenceId")]
    """Caller supplied campaign reference ID.

    If supplied, the value must be unique across all submitted campaigns. Can be
    used to prevent duplicate campaign registrations.
    """

    reseller_id: Annotated[str, PropertyInfo(alias="resellerId")]
    """
    Alphanumeric identifier of the reseller that you want to associate with this
    campaign.
    """

    sample1: str
    """Message sample. Some campaign tiers require 1 or more message samples."""

    sample2: str
    """Message sample. Some campaign tiers require 2 or more message samples."""

    sample3: str
    """Message sample. Some campaign tiers require 3 or more message samples."""

    sample4: str
    """Message sample. Some campaign tiers require 4 or more message samples."""

    sample5: str
    """Message sample. Some campaign tiers require 5 or more message samples."""

    subscriber_help: Annotated[bool, PropertyInfo(alias="subscriberHelp")]
    """Does campaign responds to help keyword(s)?"""

    subscriber_optin: Annotated[bool, PropertyInfo(alias="subscriberOptin")]
    """Does campaign require subscriber to opt-in before SMS is sent to subscriber?"""

    subscriber_optout: Annotated[bool, PropertyInfo(alias="subscriberOptout")]
    """Does campaign support subscriber opt-out keyword(s)?"""

    sub_usecases: Annotated[SequenceNotStr[str], PropertyInfo(alias="subUsecases")]
    """Campaign sub-usecases.

    Must be of defined valid sub-usecase types. Use `/registry/enum/usecase`
    operation to retrieve list of valid sub-usecases
    """

    tag: SequenceNotStr[str]
    """Tags to be set on the Campaign."""

    terms_and_conditions: Annotated[bool, PropertyInfo(alias="termsAndConditions")]
    """Is terms and conditions accepted?"""

    terms_and_conditions_link: Annotated[str, PropertyInfo(alias="termsAndConditionsLink")]
    """Link to the campaign's terms and conditions."""

    webhook_failover_url: Annotated[str, PropertyInfo(alias="webhookFailoverURL")]
    """Failover webhook to which campaign status updates are sent."""

    webhook_url: Annotated[str, PropertyInfo(alias="webhookURL")]
    """Webhook to which campaign status updates are sent."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
import time

import pytest

try:
    from nacl.signing import SigningKey
    from nacl.encoding import HexEncoder
except ImportError:
    pytest.skip("pynacl is required for webhook tests", allow_module_level=True)

from telnyx import Telnyx
from telnyx.resources.webhooks import TelnyxWebhook, TelnyxWebhookVerificationError

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestWebhooks:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    def test_method_unwrap(self, client: Telnyx) -> None:
        # Generate a test key pair for Telnyx ED25519 verification
        signing_key = SigningKey.generate()
        verify_key_hex = signing_key.verify_key.encode(encoder=HexEncoder).decode('utf-8')

        data = """{"data":{"id":"0ccc7b54-4df3-4bca-a65a-3da1ecc777f0","event_type":"call.ai_gather.ended","occurred_at":"2018-02-02T22:25:27.521992Z","payload":{"call_control_id":"v2:T02llQxIyaRkhfRKxgAP8nY511EhFLizdvdUKJiSw8d6A9BborherQ","call_leg_id":"428c31b6-7af4-4bcb-b7f5-5013ef9657c1","call_session_id":"428c31b6-7af4-4bcb-b7f5-5013ef9657c1","client_state":"aGF2ZSBhIG5pY2UgZGF5ID1d","connection_id":"7267xxxxxxxxxxxxxx","from":"+35319605860","message_history":[{"content":"Hello, can you tell me your age and where you live?","role":"assistant"},{"content":"Hello, I'm 29 and I live in Paris?","role":"user"}],"result":{"age":29,"city":"Paris"},"status":"valid","to":"+35319605860"},"record_type":"event"}}"""
        
        # Create timestamp and Telnyx-style signature
        timestamp = str(int(time.time()))
        signed_payload = timestamp.encode('utf-8') + data.encode('utf-8')
        signature = signing_key.sign(signed_payload, encoder=HexEncoder).signature.decode('utf-8')
        
        headers = {
            "Telnyx-Signature-Ed25519": signature,
            "Telnyx-Timestamp": timestamp,
            "User-Agent": "telnyx-webhooks"
        }

        try:
            result = client.webhooks.unwrap(data, headers=headers, key=verify_key_hex)
            # Verify the result is properly parsed
            assert result.data.event_type == "call.ai_gather.ended"  # type: ignore[reportUnknownMemberType]
            assert result.data.payload.call_control_id == "v2:T02llQxIyaRkhfRKxgAP8nY511EhFLizdvdUKJiSw8d6A9BborherQ"  # type: ignore[reportUnknownMemberType]
        except TelnyxWebhookVerificationError as e:
            raise AssertionError("Failed to unwrap valid webhook") from e

        # Test invalid signature
        bad_headers = [
            {**headers, "Telnyx-Signature-Ed25519": "invalid_signature"},
            {**headers, "Telnyx-Timestamp": "0"},
            {**headers, "User-Agent": "malicious-bot"},
        ]
        for bad_header in bad_headers:
            with pytest.raises(TelnyxWebhookVerificationError):
                _ = client.webhooks.unwrap(data, headers=bad_header, key=verify_key_hex)

        # Test missing headers
        incomplete_headers = {
            "Telnyx-Timestamp": timestamp,
            "User-Agent": "telnyx-webhooks"
        }
        with pytest.raises(TelnyxWebhookVerificationError):
            _ = client.webhooks.unwrap(data, headers=incomplete_headers, key=verify_key_hex)

        # Test case-insensitive headers
        lowercase_headers = {
            "telnyx-signature-ed25519": signature,
            "telnyx-timestamp": timestamp,
            "user-agent": "telnyx-webhooks"
        }
        try:
            result = client.webhooks.unwrap(data, headers=lowercase_headers, key=verify_key_hex)
            assert result.data.event_type == "call.ai_gather.ended"  # type: ignore[reportUnknownMemberType]
        except TelnyxWebhookVerificationError as e:
            raise AssertionError("Failed to unwrap webhook with lowercase headers") from e


class TestAsyncWebhooks:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    def test_method_unwrap(self, client: Telnyx) -> None:
        # Generate a test key pair for Telnyx ED25519 verification
        signing_key = SigningKey.generate()
        verify_key_hex = signing_key.verify_key.encode(encoder=HexEncoder).decode('utf-8')

        data = """{"data":{"id":"0ccc7b54-4df3-4bca-a65a-3da1ecc777f0","event_type":"call.ai_gather.ended","occurred_at":"2018-02-02T22:25:27.521992Z","payload":{"call_control_id":"v2:T02llQxIyaRkhfRKxgAP8nY511EhFLizdvdUKJiSw8d6A9BborherQ","call_leg_id":"428c31b6-7af4-4bcb-b7f5-5013ef9657c1","call_session_id":"428c31b6-7af4-4bcb-b7f5-5013ef9657c1","client_state":"aGF2ZSBhIG5pY2UgZGF5ID1d","connection_id":"7267xxxxxxxxxxxxxx","from":"+35319605860","message_history":[{"content":"Hello, can you tell me your age and where you live?","role":"assistant"},{"content":"Hello, I'm 29 and I live in Paris?","role":"user"}],"result":{"age":29,"city":"Paris"},"status":"valid","to":"+35319605860"},"record_type":"event"}}"""
        
        # Create timestamp and Telnyx-style signature
        timestamp = str(int(time.time()))
        signed_payload = timestamp.encode('utf-8') + data.encode('utf-8')
        signature = signing_key.sign(signed_payload, encoder=HexEncoder).signature.decode('utf-8')
        
        headers = {
            "Telnyx-Signature-Ed25519": signature,
            "Telnyx-Timestamp": timestamp,
            "User-Agent": "telnyx-webhooks"
        }

        try:
            result = client.webhooks.unwrap(data, headers=headers, key=verify_key_hex)
            # Verify the result is properly parsed
            assert result.data.event_type == "call.ai_gather.ended"  # type: ignore[reportUnknownMemberType]
            assert result.data.payload.call_control_id == "v2:T02llQxIyaRkhfRKxgAP8nY511EhFLizdvdUKJiSw8d6A9BborherQ"  # type: ignore[reportUnknownMemberType]
        except TelnyxWebhookVerificationError as e:
            raise AssertionError("Failed to unwrap valid webhook") from e

        # Test invalid signature
        bad_headers = [
            {**headers, "Telnyx-Signature-Ed25519": "invalid_signature"},
            {**headers, "Telnyx-Timestamp": "0"},
            {**headers, "User-Agent": "malicious-bot"},
        ]
        for bad_header in bad_headers:
            with pytest.raises(TelnyxWebhookVerificationError):
                _ = client.webhooks.unwrap(data, headers=bad_header, key=verify_key_hex)

        # Test missing headers
        incomplete_headers = {
            "Telnyx-Timestamp": timestamp,
            "User-Agent": "telnyx-webhooks"
        }
        with pytest.raises(TelnyxWebhookVerificationError):
            _ = client.webhooks.unwrap(data, headers=incomplete_headers, key=verify_key_hex)
class TestTelnyxWebhook:
    """Test the TelnyxWebhook class directly to ensure it follows standardwebhooks pattern."""

    def test_webhook_initialization(self) -> None:
        """Test that TelnyxWebhook can be initialized like standardwebhooks.Webhook."""
        signing_key = SigningKey.generate()
        verify_key_hex = signing_key.verify_key.encode(encoder=HexEncoder).decode('utf-8')
        
        # Should initialize like standardwebhooks.Webhook
        webhook = TelnyxWebhook(verify_key_hex)
        assert webhook is not None
        assert hasattr(webhook, 'verify')

    def test_webhook_verification_pattern(self) -> None:
        """Test that TelnyxWebhook.verify follows the same pattern as standardwebhooks.Webhook.verify."""
        signing_key = SigningKey.generate()
        verify_key_hex = signing_key.verify_key.encode(encoder=HexEncoder).decode('utf-8')
        
        data = """{"test": "payload"}"""
        timestamp = str(int(time.time()))
        signed_payload = timestamp.encode('utf-8') + data.encode('utf-8')
        signature = signing_key.sign(signed_payload, encoder=HexEncoder).signature.decode('utf-8')
        
        headers = {
            "Telnyx-Signature-Ed25519": signature,
            "Telnyx-Timestamp": timestamp,
            "User-Agent": "telnyx-webhooks"
        }
        
        # Should verify like standardwebhooks.Webhook.verify
        webhook = TelnyxWebhook(verify_key_hex)
        
        # Valid webhook should not raise exception
        webhook.verify(data, headers)
        
        # Invalid webhook should raise TelnyxWebhookVerificationError
        with pytest.raises(TelnyxWebhookVerificationError):
            webhook.verify(data, {**headers, "Telnyx-Signature-Ed25519": "invalid"})

    def test_exception_hierarchy(self) -> None:
        """Test that TelnyxWebhookVerificationError follows the right pattern."""
        # Should be importable like standardwebhooks.WebhookVerificationError
        assert TelnyxWebhookVerificationError is not None
        
        # Should be able to raise and catch it
        with pytest.raises(TelnyxWebhookVerificationError):
            raise TelnyxWebhookVerificationError("Test error")

    def test_key_format_handling(self) -> None:
        """Test that TelnyxWebhook handles different key formats like standardwebhooks."""
        signing_key = SigningKey.generate()
        verify_key_hex = signing_key.verify_key.encode(encoder=HexEncoder).decode('utf-8')
        verify_key_bytes = signing_key.verify_key.encode()
        
        # Should accept hex string (Telnyx format)
        webhook1 = TelnyxWebhook(verify_key_hex)
        assert webhook1 is not None
        
        # Should accept bytes (standardwebhooks format)
        webhook2 = TelnyxWebhook(verify_key_bytes)
        assert webhook2 is not None
        
        # Should reject invalid key format
        with pytest.raises(TelnyxWebhookVerificationError):
            TelnyxWebhook("invalid_key_format")


# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional

from .._models import BaseModel

__all__ = ["UsageReportListResponse", "Meta"]


class Meta(BaseModel):
    page_number: Optional[int] = None
    """Selected page number."""

    page_size: Optional[int] = None
    """Records per single page"""

    total_pages: Optional[int] = None
    """Total number of pages."""

    total_results: Optional[int] = None
    """Total number of results."""


class UsageReportListResponse(BaseModel):
    data: Optional[List[Dict[str, object]]] = None

    meta: Optional[Meta] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

from .._types import SequenceNotStr

__all__ = ["ReportListWdrsParams", "Page"]


class ReportListWdrsParams(TypedDict, total=False):
    id: str
    """WDR uuid"""

    end_date: str
    """End date"""

    imsi: str
    """International mobile subscriber identity"""

    mcc: str
    """Mobile country code"""

    mnc: str
    """Mobile network code"""

    page: Page
    """Consolidated page parameter (deepObject style).

    Originally: page[number], page[size]
    """

    phone_number: str
    """Phone number"""

    sim_card_id: str
    """Sim card unique identifier"""

    sim_group_id: str
    """Sim group unique identifier"""

    sim_group_name: str
    """Sim group name"""

    sort: SequenceNotStr[str]
    """Field used to order the data.

    If no field is specified, default value is 'created_at'
    """

    start_date: str
    """Start date"""


class Page(TypedDict, total=False):
    number: int
    """Page number"""

    size: int
    """Size of the page"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .record import Record
from .._models import BaseModel
from .interface import Interface

__all__ = ["WireguardInterfaceRetrieveResponse", "Data", "DataRegion"]


class DataRegion(BaseModel):
    code: Optional[str] = None
    """Region code of the interface."""

    name: Optional[str] = None
    """Region name of the interface."""

    record_type: Optional[str] = None
    """Identifies the type of the resource."""


class Data(Record, Interface):
    enable_sip_trunking: Optional[bool] = None
    """Enable SIP traffic forwarding over VPN interface."""

    endpoint: Optional[str] = None
    """The Telnyx WireGuard peers `Peer.endpoint` value."""

    public_key: Optional[str] = None
    """The Telnyx WireGuard peers `Peer.PublicKey`."""

    record_type: Optional[str] = None  # type: ignore
    """Identifies the type of the resource."""

    region: Optional[DataRegion] = None

    region_code: Optional[str] = None
    """The region interface is deployed to."""


class WireguardInterfaceRetrieveResponse(BaseModel):
    data: Optional[Data] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .actions import (
    ActionsResource,
    AsyncActionsResource,
    ActionsResourceWithRawResponse,
    AsyncActionsResourceWithRawResponse,
    ActionsResourceWithStreamingResponse,
    AsyncActionsResourceWithStreamingResponse,
)
from .sessions import (
    SessionsResource,
    AsyncSessionsResource,
    SessionsResourceWithRawResponse,
    AsyncSessionsResourceWithRawResponse,
    SessionsResourceWithStreamingResponse,
    AsyncSessionsResourceWithStreamingResponse,
)

__all__ = [
    "ActionsResource",
    "AsyncActionsResource",
    "ActionsResourceWithRawResponse",
    "AsyncActionsResourceWithRawResponse",
    "ActionsResourceWithStreamingResponse",
    "AsyncActionsResourceWithStreamingResponse",
    "SessionsResource",
    "AsyncSessionsResource",
    "SessionsResourceWithRawResponse",
    "AsyncSessionsResourceWithRawResponse",
    "SessionsResourceWithStreamingResponse",
    "AsyncSessionsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["ActionStartRecordingParams"]


class ActionStartRecordingParams(TypedDict, total=False):
    channels: Required[Literal["single", "dual"]]
    """
    When `dual`, final audio file will be stereo recorded with the first leg on
    channel A, and the rest on channel B.
    """

    format: Required[Literal["wav", "mp3"]]
    """The audio file format used when storing the call recording.

    Can be either `mp3` or `wav`.
    """

    client_state: str
    """Use this field to add state to every subsequent webhook.

    It must be a valid Base-64 encoded string.
    """

    command_id: str
    """Use this field to avoid duplicate commands.

    Telnyx will ignore any command with the same `command_id` for the same
    `call_control_id`.
    """

    custom_file_name: str
    """The custom recording file name to be used instead of the default `call_leg_id`.

    Telnyx will still add a Unix timestamp suffix.
    """

    max_length: int
    """Defines the maximum length for the recording in seconds.

    The minimum value is 0. The maximum value is 14400. The default value is 0
    (infinite)
    """

    play_beep: bool
    """If enabled, a beep sound will be played at the start of a recording."""

    recording_track: Literal["both", "inbound", "outbound"]
    """The audio track to be recorded.

    Can be either `both`, `inbound` or `outbound`. If only single track is specified
    (`inbound`, `outbound`), `channels` configuration is ignored and it will be
    recorded as mono (single channel).
    """

    timeout_secs: int
    """
    The number of seconds that Telnyx will wait for the recording to be stopped if
    silence is detected. The timer only starts when the speech is detected. Please
    note that call transcription is used to detect silence and the related charge
    will be applied. The minimum value is 0. The default value is 0 (infinite)
    """

    transcription: bool
    """Enable post recording transcription. The default value is false."""

    transcription_engine: str
    """Engine to use for speech recognition. `A` - `Google`"""

    transcription_language: Literal[
        "af-ZA",
        "am-ET",
        "ar-AE",
        "ar-BH",
        "ar-DZ",
        "ar-EG",
        "ar-IL",
        "ar-IQ",
        "ar-JO",
        "ar-KW",
        "ar-LB",
        "ar-MA",
        "ar-MR",
        "ar-OM",
        "ar-PS",
        "ar-QA",
        "ar-SA",
        "ar-TN",
        "ar-YE",
        "az-AZ",
        "bg-BG",
        "bn-BD",
        "bn-IN",
        "bs-BA",
        "ca-ES",
        "cs-CZ",
        "da-DK",
        "de-AT",
        "de-CH",
        "de-DE",
        "el-GR",
        "en-AU",
        "en-CA",
        "en-GB",
        "en-GH",
        "en-HK",
        "en-IE",
        "en-IN",
        "en-KE",
        "en-NG",
        "en-NZ",
        "en-PH",
        "en-PK",
        "en-SG",
        "en-TZ",
        "en-US",
        "en-ZA",
        "es-AR",
        "es-BO",
        "es-CL",
        "es-CO",
        "es-CR",
        "es-DO",
        "es-EC",
        "es-ES",
        "es-GT",
        "es-HN",
        "es-MX",
        "es-NI",
        "es-PA",
        "es-PE",
        "es-PR",
        "es-PY",
        "es-SV",
        "es-US",
        "es-UY",
        "es-VE",
        "et-EE",
        "eu-ES",
        "fa-IR",
        "fi-FI",
        "fil-PH",
        "fr-BE",
        "fr-CA",
        "fr-CH",
        "fr-FR",
        "gl-ES",
        "gu-IN",
        "hi-IN",
        "hr-HR",
        "hu-HU",
        "hy-AM",
        "id-ID",
        "is-IS",
        "it-CH",
        "it-IT",
        "iw-IL",
        "ja-JP",
        "jv-ID",
        "ka-GE",
        "kk-KZ",
        "km-KH",
        "kn-IN",
        "ko-KR",
        "lo-LA",
        "lt-LT",
        "lv-LV",
        "mk-MK",
        "ml-IN",
        "mn-MN",
        "mr-IN",
        "ms-MY",
        "my-MM",
        "ne-NP",
        "nl-BE",
        "nl-NL",
        "no-NO",
        "pa-Guru-IN",
        "pl-PL",
        "pt-BR",
        "pt-PT",
        "ro-RO",
        "ru-RU",
        "rw-RW",
        "si-LK",
        "sk-SK",
        "sl-SI",
        "sq-AL",
        "sr-RS",
        "ss-latn-za",
        "st-ZA",
        "su-ID",
        "sv-SE",
        "sw-KE",
        "sw-TZ",
        "ta-IN",
        "ta-LK",
        "ta-MY",
        "ta-SG",
        "te-IN",
        "th-TH",
        "tn-latn-za",
        "tr-TR",
        "ts-ZA",
        "uk-UA",
        "ur-IN",
        "ur-PK",
        "uz-UZ",
        "ve-ZA",
        "vi-VN",
        "xh-ZA",
        "yue-Hant-HK",
        "zh",
        "zh-TW",
        "zu-ZA",
    ]
    """Language to use for speech recognition"""

    transcription_max_speaker_count: int
    """Defines maximum number of speakers in the conversation.

    Applies to `google` engine only.
    """

    transcription_min_speaker_count: int
    """Defines minimum number of speakers in the conversation.

    Applies to `google` engine only.
    """

    transcription_profanity_filter: bool
    """Enables profanity_filter. Applies to `google` engine only."""

    transcription_speaker_diarization: bool
    """Enables speaker diarization. Applies to `google` engine only."""

    trim: Literal["trim-silence"]
    """
    When set to `trim-silence`, silence will be removed from the beginning and end
    of the recording.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types import (
    NumberOrderListResponse,
    NumberOrderCreateResponse,
    NumberOrderUpdateResponse,
    NumberOrderRetrieveResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestNumberOrders:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: Telnyx) -> None:
        number_order = client.number_orders.create()
        assert_matches_type(NumberOrderCreateResponse, number_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Telnyx) -> None:
        number_order = client.number_orders.create(
            billing_group_id="abc85f64-5717-4562-b3fc-2c9600",
            connection_id="346789098765567",
            customer_reference="MY REF 001",
            messaging_profile_id="abc85f64-5717-4562-b3fc-2c9600",
            phone_numbers=[
                {
                    "phone_number": "+19705555098",
                    "bundle_id": "bc8e4d67-33a0-4cbb-af74-7b58f05bd494",
                    "requirement_group_id": "dbbd94ee-9079-488f-80ba-f566b247fd7",
                },
                {
                    "phone_number": "+492111609539",
                    "bundle_id": "bc8e4d67-33a0-4cbb-af74-7b58f05bd494",
                    "requirement_group_id": "dbbd94ee-9079-488f-80ba-f566b247fd79",
                },
            ],
        )
        assert_matches_type(NumberOrderCreateResponse, number_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Telnyx) -> None:
        response = client.number_orders.with_raw_response.create()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        number_order = response.parse()
        assert_matches_type(NumberOrderCreateResponse, number_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Telnyx) -> None:
        with client.number_orders.with_streaming_response.create() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            number_order = response.parse()
            assert_matches_type(NumberOrderCreateResponse, number_order, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        number_order = client.number_orders.retrieve(
            "number_order_id",
        )
        assert_matches_type(NumberOrderRetrieveResponse, number_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.number_orders.with_raw_response.retrieve(
            "number_order_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        number_order = response.parse()
        assert_matches_type(NumberOrderRetrieveResponse, number_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.number_orders.with_streaming_response.retrieve(
            "number_order_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            number_order = response.parse()
            assert_matches_type(NumberOrderRetrieveResponse, number_order, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `number_order_id` but received ''"):
            client.number_orders.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update(self, client: Telnyx) -> None:
        number_order = client.number_orders.update(
            number_order_id="number_order_id",
        )
        assert_matches_type(NumberOrderUpdateResponse, number_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: Telnyx) -> None:
        number_order = client.number_orders.update(
            number_order_id="number_order_id",
            customer_reference="MY REF 001",
            regulatory_requirements=[
                {
                    "field_value": "45f45a04-b4be-4592-95b1-9306b9db2b21",
                    "requirement_id": "8ffb3622-7c6b-4ccc-b65f-7a3dc0099576",
                }
            ],
        )
        assert_matches_type(NumberOrderUpdateResponse, number_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Telnyx) -> None:
        response = client.number_orders.with_raw_response.update(
            number_order_id="number_order_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        number_order = response.parse()
        assert_matches_type(NumberOrderUpdateResponse, number_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Telnyx) -> None:
        with client.number_orders.with_streaming_response.update(
            number_order_id="number_order_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            number_order = response.parse()
            assert_matches_type(NumberOrderUpdateResponse, number_order, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `number_order_id` but received ''"):
            client.number_orders.with_raw_response.update(
                number_order_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        number_order = client.number_orders.list()
        assert_matches_type(NumberOrderListResponse, number_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        number_order = client.number_orders.list(
            filter={
                "created_at": {
                    "gt": "gt",
                    "lt": "lt",
                },
                "customer_reference": "customer_reference",
                "phone_numbers_count": "phone_numbers_count",
                "requirements_met": True,
                "status": "status",
            },
            page={
                "number": 1,
                "size": 1,
            },
        )
        assert_matches_type(NumberOrderListResponse, number_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.number_orders.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        number_order = response.parse()
        assert_matches_type(NumberOrderListResponse, number_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.number_orders.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            number_order = response.parse()
            assert_matches_type(NumberOrderListResponse, number_order, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncNumberOrders:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncTelnyx) -> None:
        number_order = await async_client.number_orders.create()
        assert_matches_type(NumberOrderCreateResponse, number_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncTelnyx) -> None:
        number_order = await async_client.number_orders.create(
            billing_group_id="abc85f64-5717-4562-b3fc-2c9600",
            connection_id="346789098765567",
            customer_reference="MY REF 001",
            messaging_profile_id="abc85f64-5717-4562-b3fc-2c9600",
            phone_numbers=[
                {
                    "phone_number": "+19705555098",
                    "bundle_id": "bc8e4d67-33a0-4cbb-af74-7b58f05bd494",
                    "requirement_group_id": "dbbd94ee-9079-488f-80ba-f566b247fd7",
                },
                {
                    "phone_number": "+492111609539",
                    "bundle_id": "bc8e4d67-33a0-4cbb-af74-7b58f05bd494",
                    "requirement_group_id": "dbbd94ee-9079-488f-80ba-f566b247fd79",
                },
            ],
        )
        assert_matches_type(NumberOrderCreateResponse, number_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.number_orders.with_raw_response.create()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        number_order = await response.parse()
        assert_matches_type(NumberOrderCreateResponse, number_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncTelnyx) -> None:
        async with async_client.number_orders.with_streaming_response.create() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            number_order = await response.parse()
            assert_matches_type(NumberOrderCreateResponse, number_order, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        number_order = await async_client.number_orders.retrieve(
            "number_order_id",
        )
        assert_matches_type(NumberOrderRetrieveResponse, number_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.number_orders.with_raw_response.retrieve(
            "number_order_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        number_order = await response.parse()
        assert_matches_type(NumberOrderRetrieveResponse, number_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.number_orders.with_streaming_response.retrieve(
            "number_order_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            number_order = await response.parse()
            assert_matches_type(NumberOrderRetrieveResponse, number_order, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `number_order_id` but received ''"):
            await async_client.number_orders.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncTelnyx) -> None:
        number_order = await async_client.number_orders.update(
            number_order_id="number_order_id",
        )
        assert_matches_type(NumberOrderUpdateResponse, number_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncTelnyx) -> None:
        number_order = await async_client.number_orders.update(
            number_order_id="number_order_id",
            customer_reference="MY REF 001",
            regulatory_requirements=[
                {
                    "field_value": "45f45a04-b4be-4592-95b1-9306b9db2b21",
                    "requirement_id": "8ffb3622-7c6b-4ccc-b65f-7a3dc0099576",
                }
            ],
        )
        assert_matches_type(NumberOrderUpdateResponse, number_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.number_orders.with_raw_response.update(
            number_order_id="number_order_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        number_order = await response.parse()
        assert_matches_type(NumberOrderUpdateResponse, number_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncTelnyx) -> None:
        async with async_client.number_orders.with_streaming_response.update(
            number_order_id="number_order_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            number_order = await response.parse()
            assert_matches_type(NumberOrderUpdateResponse, number_order, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `number_order_id` but received ''"):
            await async_client.number_orders.with_raw_response.update(
                number_order_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        number_order = await async_client.number_orders.list()
        assert_matches_type(NumberOrderListResponse, number_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        number_order = await async_client.number_orders.list(
            filter={
                "created_at": {
                    "gt": "gt",
                    "lt": "lt",
                },
                "customer_reference": "customer_reference",
                "phone_numbers_count": "phone_numbers_count",
                "requirements_met": True,
                "status": "status",
            },
            page={
                "number": 1,
                "size": 1,
            },
        )
        assert_matches_type(NumberOrderListResponse, number_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.number_orders.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        number_order = await response.parse()
        assert_matches_type(NumberOrderListResponse, number_order, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.number_orders.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            number_order = await response.parse()
            assert_matches_type(NumberOrderListResponse, number_order, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["BrandQualifyByUsecaseResponse"]


class BrandQualifyByUsecaseResponse(BaseModel):
    annual_fee: Optional[float] = FieldInfo(alias="annualFee", default=None)
    """Campaign annual subscription fee"""

    max_sub_usecases: Optional[int] = FieldInfo(alias="maxSubUsecases", default=None)
    """Maximum number of sub-usecases declaration required."""

    min_sub_usecases: Optional[int] = FieldInfo(alias="minSubUsecases", default=None)
    """Minimum number of sub-usecases declaration required."""

    mno_metadata: Optional[Dict[str, object]] = FieldInfo(alias="mnoMetadata", default=None)
    """Map of usecase metadata for each MNO.

    Key is the network ID of the MNO (e.g. 10017), Value is the mno metadata for the
    usecase.
    """

    monthly_fee: Optional[float] = FieldInfo(alias="monthlyFee", default=None)
    """Campaign monthly subscription fee"""

    quarterly_fee: Optional[float] = FieldInfo(alias="quarterlyFee", default=None)
    """Campaign quarterly subscription fee"""

    usecase: Optional[str] = None
    """Campaign usecase"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import date
from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["RoomListParams", "Filter", "FilterDateCreatedAt", "FilterDateUpdatedAt", "Page"]


class RoomListParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter (deepObject style).

    Originally: filter[date_created_at][eq], filter[date_created_at][gte],
    filter[date_created_at][lte], filter[date_updated_at][eq],
    filter[date_updated_at][gte], filter[date_updated_at][lte], filter[unique_name]
    """

    include_sessions: bool
    """To decide if room sessions should be included in the response."""

    page: Page
    """Consolidated page parameter (deepObject style).

    Originally: page[size], page[number]
    """


class FilterDateCreatedAt(TypedDict, total=False):
    eq: Annotated[Union[str, date], PropertyInfo(format="iso8601")]
    """ISO 8601 date for filtering rooms created on that date."""

    gte: Annotated[Union[str, date], PropertyInfo(format="iso8601")]
    """ISO 8601 date for filtering rooms created on or after that date."""

    lte: Annotated[Union[str, date], PropertyInfo(format="iso8601")]
    """ISO 8601 date for filtering rooms created on or before that date."""


class FilterDateUpdatedAt(TypedDict, total=False):
    eq: Annotated[Union[str, date], PropertyInfo(format="iso8601")]
    """ISO 8601 date for filtering rooms updated on that date."""

    gte: Annotated[Union[str, date], PropertyInfo(format="iso8601")]
    """ISO 8601 date for filtering rooms updated on or after that date."""

    lte: Annotated[Union[str, date], PropertyInfo(format="iso8601")]
    """ISO 8601 date for filtering rooms updated on or before that date."""


class Filter(TypedDict, total=False):
    date_created_at: FilterDateCreatedAt

    date_updated_at: FilterDateUpdatedAt

    unique_name: str
    """Unique_name for filtering rooms."""


class Page(TypedDict, total=False):
    number: int
    """The page number to load."""

    size: int
    """The size of the page."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Iterable
from typing_extensions import Literal, Required, TypedDict

from ..._types import SequenceNotStr

__all__ = ["InferenceEmbeddingWebhookToolParamsParam", "BodyParameters", "Header", "PathParameters", "QueryParameters"]


class BodyParameters(TypedDict, total=False):
    properties: Dict[str, object]
    """The properties of the body parameters."""

    required: SequenceNotStr[str]
    """The required properties of the body parameters."""

    type: Literal["object"]


class Header(TypedDict, total=False):
    name: str

    value: str
    """The value of the header.

    Note that we support mustache templating for the value. For example you can use
    `Bearer {{#integration_secret}}test-secret{{/integration_secret}}` to pass the
    value of the integration secret as the bearer token.
    [Telnyx signature headers](https://developers.telnyx.com/docs/voice/programmable-voice/voice-api-webhooks)
    will be automatically added to the request.
    """


class PathParameters(TypedDict, total=False):
    properties: Dict[str, object]
    """The properties of the path parameters."""

    required: SequenceNotStr[str]
    """The required properties of the path parameters."""

    type: Literal["object"]


class QueryParameters(TypedDict, total=False):
    properties: Dict[str, object]
    """The properties of the query parameters."""

    required: SequenceNotStr[str]
    """The required properties of the query parameters."""

    type: Literal["object"]


class InferenceEmbeddingWebhookToolParamsParam(TypedDict, total=False):
    description: Required[str]
    """The description of the tool."""

    name: Required[str]
    """The name of the tool."""

    url: Required[str]
    """The URL of the external tool to be called.

    This URL is going to be used by the assistant. The URL can be templated like:
    `https://example.com/api/v1/{id}`, where `{id}` is a placeholder for a value
    that will be provided by the assistant if `path_parameters` are provided with
    the `id` attribute.
    """

    body_parameters: BodyParameters
    """The body parameters the webhook tool accepts, described as a JSON Schema object.

    These parameters will be passed to the webhook as the body of the request. See
    the [JSON Schema reference](https://json-schema.org/understanding-json-schema)
    for documentation about the format
    """

    headers: Iterable[Header]
    """The headers to be sent to the external tool."""

    method: Literal["GET", "POST", "PUT", "DELETE", "PATCH"]
    """The HTTP method to be used when calling the external tool."""

    path_parameters: PathParameters
    """The path parameters the webhook tool accepts, described as a JSON Schema object.

    These parameters will be passed to the webhook as the path of the request if the
    URL contains a placeholder for a value. See the
    [JSON Schema reference](https://json-schema.org/understanding-json-schema) for
    documentation about the format
    """

    query_parameters: QueryParameters
    """The query parameters the webhook tool accepts, described as a JSON Schema
    object.

    These parameters will be passed to the webhook as the query of the request. See
    the [JSON Schema reference](https://json-schema.org/understanding-json-schema)
    for documentation about the format
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types import (
    PhoneNumberAssignmentByProfileAssignResponse,
    PhoneNumberAssignmentByProfileRetrieveStatusResponse,
    PhoneNumberAssignmentByProfileRetrievePhoneNumberStatusResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestPhoneNumberAssignmentByProfile:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_assign(self, client: Telnyx) -> None:
        phone_number_assignment_by_profile = client.phone_number_assignment_by_profile.assign(
            messaging_profile_id="4001767e-ce0f-4cae-9d5f-0d5e636e7809",
        )
        assert_matches_type(
            PhoneNumberAssignmentByProfileAssignResponse, phone_number_assignment_by_profile, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_assign_with_all_params(self, client: Telnyx) -> None:
        phone_number_assignment_by_profile = client.phone_number_assignment_by_profile.assign(
            messaging_profile_id="4001767e-ce0f-4cae-9d5f-0d5e636e7809",
            campaign_id="4b300178-131c-d902-d54e-72d90ba1620j",
            tcr_campaign_id="CWZTFH1",
        )
        assert_matches_type(
            PhoneNumberAssignmentByProfileAssignResponse, phone_number_assignment_by_profile, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_assign(self, client: Telnyx) -> None:
        response = client.phone_number_assignment_by_profile.with_raw_response.assign(
            messaging_profile_id="4001767e-ce0f-4cae-9d5f-0d5e636e7809",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number_assignment_by_profile = response.parse()
        assert_matches_type(
            PhoneNumberAssignmentByProfileAssignResponse, phone_number_assignment_by_profile, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_assign(self, client: Telnyx) -> None:
        with client.phone_number_assignment_by_profile.with_streaming_response.assign(
            messaging_profile_id="4001767e-ce0f-4cae-9d5f-0d5e636e7809",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number_assignment_by_profile = response.parse()
            assert_matches_type(
                PhoneNumberAssignmentByProfileAssignResponse, phone_number_assignment_by_profile, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve_phone_number_status(self, client: Telnyx) -> None:
        phone_number_assignment_by_profile = client.phone_number_assignment_by_profile.retrieve_phone_number_status(
            task_id="taskId",
        )
        assert_matches_type(
            PhoneNumberAssignmentByProfileRetrievePhoneNumberStatusResponse,
            phone_number_assignment_by_profile,
            path=["response"],
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve_phone_number_status_with_all_params(self, client: Telnyx) -> None:
        phone_number_assignment_by_profile = client.phone_number_assignment_by_profile.retrieve_phone_number_status(
            task_id="taskId",
            page=0,
            records_per_page=0,
        )
        assert_matches_type(
            PhoneNumberAssignmentByProfileRetrievePhoneNumberStatusResponse,
            phone_number_assignment_by_profile,
            path=["response"],
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve_phone_number_status(self, client: Telnyx) -> None:
        response = client.phone_number_assignment_by_profile.with_raw_response.retrieve_phone_number_status(
            task_id="taskId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number_assignment_by_profile = response.parse()
        assert_matches_type(
            PhoneNumberAssignmentByProfileRetrievePhoneNumberStatusResponse,
            phone_number_assignment_by_profile,
            path=["response"],
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve_phone_number_status(self, client: Telnyx) -> None:
        with client.phone_number_assignment_by_profile.with_streaming_response.retrieve_phone_number_status(
            task_id="taskId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number_assignment_by_profile = response.parse()
            assert_matches_type(
                PhoneNumberAssignmentByProfileRetrievePhoneNumberStatusResponse,
                phone_number_assignment_by_profile,
                path=["response"],
            )

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve_phone_number_status(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `task_id` but received ''"):
            client.phone_number_assignment_by_profile.with_raw_response.retrieve_phone_number_status(
                task_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve_status(self, client: Telnyx) -> None:
        phone_number_assignment_by_profile = client.phone_number_assignment_by_profile.retrieve_status(
            "taskId",
        )
        assert_matches_type(
            PhoneNumberAssignmentByProfileRetrieveStatusResponse, phone_number_assignment_by_profile, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve_status(self, client: Telnyx) -> None:
        response = client.phone_number_assignment_by_profile.with_raw_response.retrieve_status(
            "taskId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number_assignment_by_profile = response.parse()
        assert_matches_type(
            PhoneNumberAssignmentByProfileRetrieveStatusResponse, phone_number_assignment_by_profile, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve_status(self, client: Telnyx) -> None:
        with client.phone_number_assignment_by_profile.with_streaming_response.retrieve_status(
            "taskId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number_assignment_by_profile = response.parse()
            assert_matches_type(
                PhoneNumberAssignmentByProfileRetrieveStatusResponse,
                phone_number_assignment_by_profile,
                path=["response"],
            )

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve_status(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `task_id` but received ''"):
            client.phone_number_assignment_by_profile.with_raw_response.retrieve_status(
                "",
            )


class TestAsyncPhoneNumberAssignmentByProfile:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_assign(self, async_client: AsyncTelnyx) -> None:
        phone_number_assignment_by_profile = await async_client.phone_number_assignment_by_profile.assign(
            messaging_profile_id="4001767e-ce0f-4cae-9d5f-0d5e636e7809",
        )
        assert_matches_type(
            PhoneNumberAssignmentByProfileAssignResponse, phone_number_assignment_by_profile, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_assign_with_all_params(self, async_client: AsyncTelnyx) -> None:
        phone_number_assignment_by_profile = await async_client.phone_number_assignment_by_profile.assign(
            messaging_profile_id="4001767e-ce0f-4cae-9d5f-0d5e636e7809",
            campaign_id="4b300178-131c-d902-d54e-72d90ba1620j",
            tcr_campaign_id="CWZTFH1",
        )
        assert_matches_type(
            PhoneNumberAssignmentByProfileAssignResponse, phone_number_assignment_by_profile, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_assign(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.phone_number_assignment_by_profile.with_raw_response.assign(
            messaging_profile_id="4001767e-ce0f-4cae-9d5f-0d5e636e7809",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number_assignment_by_profile = await response.parse()
        assert_matches_type(
            PhoneNumberAssignmentByProfileAssignResponse, phone_number_assignment_by_profile, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_assign(self, async_client: AsyncTelnyx) -> None:
        async with async_client.phone_number_assignment_by_profile.with_streaming_response.assign(
            messaging_profile_id="4001767e-ce0f-4cae-9d5f-0d5e636e7809",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number_assignment_by_profile = await response.parse()
            assert_matches_type(
                PhoneNumberAssignmentByProfileAssignResponse, phone_number_assignment_by_profile, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve_phone_number_status(self, async_client: AsyncTelnyx) -> None:
        phone_number_assignment_by_profile = (
            await async_client.phone_number_assignment_by_profile.retrieve_phone_number_status(
                task_id="taskId",
            )
        )
        assert_matches_type(
            PhoneNumberAssignmentByProfileRetrievePhoneNumberStatusResponse,
            phone_number_assignment_by_profile,
            path=["response"],
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve_phone_number_status_with_all_params(self, async_client: AsyncTelnyx) -> None:
        phone_number_assignment_by_profile = (
            await async_client.phone_number_assignment_by_profile.retrieve_phone_number_status(
                task_id="taskId",
                page=0,
                records_per_page=0,
            )
        )
        assert_matches_type(
            PhoneNumberAssignmentByProfileRetrievePhoneNumberStatusResponse,
            phone_number_assignment_by_profile,
            path=["response"],
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve_phone_number_status(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.phone_number_assignment_by_profile.with_raw_response.retrieve_phone_number_status(
            task_id="taskId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number_assignment_by_profile = await response.parse()
        assert_matches_type(
            PhoneNumberAssignmentByProfileRetrievePhoneNumberStatusResponse,
            phone_number_assignment_by_profile,
            path=["response"],
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve_phone_number_status(self, async_client: AsyncTelnyx) -> None:
        async with async_client.phone_number_assignment_by_profile.with_streaming_response.retrieve_phone_number_status(
            task_id="taskId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number_assignment_by_profile = await response.parse()
            assert_matches_type(
                PhoneNumberAssignmentByProfileRetrievePhoneNumberStatusResponse,
                phone_number_assignment_by_profile,
                path=["response"],
            )

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve_phone_number_status(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `task_id` but received ''"):
            await async_client.phone_number_assignment_by_profile.with_raw_response.retrieve_phone_number_status(
                task_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve_status(self, async_client: AsyncTelnyx) -> None:
        phone_number_assignment_by_profile = await async_client.phone_number_assignment_by_profile.retrieve_status(
            "taskId",
        )
        assert_matches_type(
            PhoneNumberAssignmentByProfileRetrieveStatusResponse, phone_number_assignment_by_profile, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve_status(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.phone_number_assignment_by_profile.with_raw_response.retrieve_status(
            "taskId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number_assignment_by_profile = await response.parse()
        assert_matches_type(
            PhoneNumberAssignmentByProfileRetrieveStatusResponse, phone_number_assignment_by_profile, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve_status(self, async_client: AsyncTelnyx) -> None:
        async with async_client.phone_number_assignment_by_profile.with_streaming_response.retrieve_status(
            "taskId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number_assignment_by_profile = await response.parse()
            assert_matches_type(
                PhoneNumberAssignmentByProfileRetrieveStatusResponse,
                phone_number_assignment_by_profile,
                path=["response"],
            )

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve_status(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `task_id` but received ''"):
            await async_client.phone_number_assignment_by_profile.with_raw_response.retrieve_status(
                "",
            )
